;;; notmuch-version.el --- version of notmuch  -*- emacs-lisp -*-
;;
;; Generated file (from notmuch-version.el.tmpl) -- do not edit!
;;
;; This file is part of Notmuch.
;;
;; Notmuch is free software: you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;;
;; Notmuch is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with Notmuch.  If not, see <https://www.gnu.org/licenses/>.

;;; Code:

(defconst notmuch-emacs-version "0.40"
  "Version of Notmuch Emacs MUA.")

(provide 'notmuch-version)

;;; notmuch-version.el ends here
